/* eslint-disable react/jsx-max-props-per-line */
/* eslint-disable max-len */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { Menu } from 'primereact/menu';
import { Button } from 'primereact/button';
import { Dialog } from 'primereact/dialog';
import { ProgressSpinner } from 'primereact/progressspinner';
import ActionButton from './ActionButton';
import BaseService from './../baseContainers/BaseService';

class EpuapService extends BaseService {
	constructor(domain) {
		super(domain);
		this.path = 'epuap';
	}

	addDocument(data) {
		return this.fetch(`${this.domain}/${this.path}/addDocument`, {
			method: 'POST',
			body: JSON.stringify(data),
		}).then(res => {
			return Promise.resolve(res);
		});
	}

	getEpuapData(epuapResponseId) {
		return this.fetch(`${this.domain}/${this.path}/details/${epuapResponseId}`, {
			method: 'GET',
		}).then(res => {
			return Promise.resolve(res);
		});
	}
}

class SignButtonComponent extends Component {
	constructor(props) {
		super(props);
		this.service = new EpuapService(props.baseRestApiUrl);
		const items = [];
		const epuapOptionEnabled = props.options.indexOf('EPUAP') >= 0;
		const certOptionEnabled = props.options.indexOf('CERT') >= 0;
		if (certOptionEnabled) {
			items.push({
				label: 'Podpisz certyfikatem kwalifikowanym',
				icon: 'pi',
				command: e => this.signQualifyCerificate(e),
			});
		}
		if (epuapOptionEnabled) {
			items.push({
				label: 'Podpisz profilem EPUAP',
				icon: 'pi',
				command: () => {
					this.signEPUAP();
				},
			});
		}
		this.state = {
			processDialogVisible: this.isRedirectFromEpuap(),
			items,
		};
		this.handleSign = this.handleSign.bind(this);
		this.signQualifyCerificate = this.signQualifyCerificate.bind(this);
		this.signEPUAP = this.signEPUAP.bind(this);
		this.menuRef = React.createRef();
	}

	componentDidMount() {
		// eslint-disable-next-line no-undef
		$('a.p-menuitem-link').removeAttr('href');
		const { epuapResult, epuapResponseId, handleEpuapResult, onDataSigned, onError } = this.props;
		console.log(
			'SignButton:componentDidMount, this.props.epuapResult:',
			epuapResult,
			', this.props.epuapResponseId:',
			epuapResponseId
		);
		if (this.isRedirectFromEpuap()) {
			this.service
				.getEpuapData(epuapResponseId)
				.then(epuapData => {
					// const elementB64 = btoa(unescape(encodeURIComponent(JSON.stringify(this.props.element))));
					const element = JSON.parse(decodeURIComponent(escape(atob(epuapData.cachedData))));
					let signatureB64 = epuapData.signatureB64;
					signatureB64 = this.normalizeB64Data(signatureB64);
					console.log('dataFromEpuap: element:', element, ', signatureB64:', signatureB64);
					if (epuapData.success === true) {
						if (handleEpuapResult !== undefined) {
							handleEpuapResult(element, signatureB64);

							onDataSigned(signatureB64, this.hideProcessDialog.bind(this));
						} else {
							this.hideProcessDialog();
						}
					} else {
						this.hideProcessDialog();
						onError('Wystąpił nieoczekiwany błąd podpisu EPUAP');
					}
				})
				.catch(err => {
					console.log('error:', err);
					onError('Wystąpił nieoczekiwany błąd podpisu EPUAP');
					this.hideProcessDialog();
				});
		}
	}

	componentDidUpdate() {
		// eslint-disable-next-line no-undef
		$('a.p-menuitem-link').removeAttr('href');
	}

	isRedirectFromEpuap() {
		const { epuapResult, epuapResponseId } = this.props;
		if ((epuapResult === 'success' || epuapResult === 'fail') && epuapResponseId !== undefined) {
			return true;
		} else {
			return false;
		}
	}

	signQualifyCerificate(e) {
		const { asyncMode, asyncPrepareDataToSign, dataToSignB64, onError } = this.props;
		console.log('signQualifyCerificate, asyncMode:', asyncMode);
		if (e && e.originalEvent) {
			e.originalEvent.preventDefault();
		}
		let dataToSignBase64 = undefined;
		if (asyncMode) {
			this.setState({ processDialogVisible: true });
			const promise = asyncPrepareDataToSign();
			if (promise === undefined) {
				this.hideProcessDialog();
				return;
			}
			promise
				.then(data => {
					console.log('signQualifyCerificate, dataToSign:', data.dataToSign);
					dataToSignBase64 = data.dataToSign;
					this.setState({ dataToSignB64: dataToSignBase64 }, () => this.configureSzafir());
				})
				.catch(e => {
					this.hideProcessDialog();
					onError('Błąd generowania danych do podpisu', e);
				});
		} else {
			dataToSignBase64 = dataToSignB64;
			this.setState(
				{
					processDialogVisible: true,
					dataToSignB64: dataToSignBase64,
				},
				() => this.configureSzafir()
			);
		}
	}

	configureSzafir() {
		console.log('configureSzafir...');
		const { debug, devMode, devSzafirUri } = this.props;
		// eslint-disable-next-line no-undef
		const SZAFIR = SZAFIR_SDK;

		SZAFIR.config({
			sdk_location_url: devMode ? `${devSzafirUri}/o/rest-api/szafir/` : '/o/rest-api/szafir/',
			versions_file: 'versions.xml',
			document_dependencies_folder: 'somewhere',
			series_authorization: true,
			onError: err => this.szafirErrorCallback(err),
			onErrorNoExtension: () => this.handleSzafirErrorNoExtension(),
			onErrorNoHostApp: () => this.handleSzafirErrorNoHostApp(),
			debug,
		});
		console.log('SZAFIR_SDK.version:', SZAFIR.version);
		console.log('SZAFIR_SDK.initialized', SZAFIR.isExtInitialized());
		this.initSzafirApplet();
		this.userLoadSzafir();
	}

	userLoadSzafir() {
		console.log('Start loading SZAFIR MODULE...');
		// eslint-disable-next-line no-undef
		SZAFIR_SDK.loadSzafir(() => this.szafirStartComponentURI());
	}

	szafirStartComponentURI() {
		const { devMode, devSzafirUri } = this.props;
		console.log('Start loading SZAFIR SETTINGS...4');
		// eslint-disable-next-line no-undef
		if ($('#SZAFIRSDK_APPLET_OBJECT').length > 0) {
			console.log('SZAFIR_SDK.mode: APPLET');
		} else {
			console.log('SZAFIR_SDK.mode: WEB-MODULE');
		}
		// eslint-disable-next-line max-len
		let settingsUri =
			'/o/rest-api/szafir/settings_files/settings_cryptoproviders_wszystkie_centra.xml';
		if (devMode) {
			settingsUri = `${devSzafirUri}${settingsUri}`;
		}
		// eslint-disable-next-line no-undef
		SZAFIR_SDK.startComponentURI(settingsUri, () => this.szafirDoTaskList());
	}

	szafirDoTaskList() {
		const { dataToSignB64 } = this.state;
		console.log('szafirDoTaskList...');
		const taskListXml =
			`${'<TaskList>' +
				'<SigningTask>' +
				'    <Format>EPO-XAdES-BES</Format>' +
				'    <DataEmbedding>true</DataEmbedding>' +
				'    <SigningTaskItem Id="tb3bb4a8d-91d8-4928-89e5-bbb088025e94">' +
				'        <DataToSign>' +
				'            <Base64Binary>'}${dataToSignB64}</Base64Binary>` +
			'        </DataToSign>' +
			'        <SignatureToCreate>' +
			'            <Base64Binary/>' +
			'        </SignatureToCreate>' +
			'    </SigningTaskItem>' +
			'</SigningTask>' +
			'</TaskList>';

		console.log('taskListXml:', taskListXml);
		// eslint-disable-next-line no-undef
		SZAFIR_SDK.doTaskList(taskListXml, result => this.onSignatureCreated(result));
	}

	userUnloadSzafir() {
		console.log('Start unloading SZAFIR SDK...');
		// eslint-disable-next-line no-undef
		SZAFIR_SDK.unloadSzafir(() => console.log('szafir has been unloaded...'));
	}

	normalizeB64Data(data) {
		try {
			return data.replace(/(\r\n|\n|\r)/gm, '');
		} catch (err) {
			return data;
		}
	}

	onSignatureCreated(result) {
		const { onDataSigned } = this.props;
		console.log('onSignatureCreated: result:', result);
		// signButtonDisable(false);
		this.userUnloadSzafir();
		const parser = new DOMParser();
		const xmlDoc = parser.parseFromString(result, 'text/xml');
		const codePath = '/TaskList/SigningTask/SigningTaskItem[1]/SigningResult/Result/@code';
		let nodes = xmlDoc.evaluate(codePath, xmlDoc, null, XPathResult.ANY_TYPE, null);
		const code = nodes.iterateNext().nodeValue;
		const descPath = '/TaskList/SigningTask/SigningTaskItem[1]/SigningResult/Result/text()';
		nodes = xmlDoc.evaluate(descPath, xmlDoc, null, XPathResult.ANY_TYPE, null);
		const desc = nodes.iterateNext().nodeValue;
		console.log('code:', code, '', 'desc:', desc);
		if (parseInt(code) === 0) {
			const sigPath =
				'/TaskList/SigningTask/SigningTaskItem[1]/SignatureToCreate/Base64Binary/text()';
			nodes = xmlDoc.evaluate(sigPath, xmlDoc, null, XPathResult.ANY_TYPE, null);
			let sigB64 = nodes.iterateNext().nodeValue;
			// this.hideProcessDialog();
			sigB64 = this.normalizeB64Data(sigB64);
			onDataSigned(sigB64, this.hideProcessDialog.bind(this));
		} else {
			console.log('doTaskList error...');
			this.hideProcessDialog();
		}
	}

	handleSzafirErrorNoExtension() {
		console.log('handleSzafirErrorNoExtension...');
		this.setState({ noExtDialogVisible: true });
		this.hideProcessDialog();
	}

	handleSzafirErrorNoHostApp() {
		console.log('handleSzafirErrorNoHostApp...');
		this.setState({ noHostDialogVisible: true });
		this.hideProcessDialog();
	}

	hideProcessDialog(e) {
		if (e !== undefined) {
			e.preventDefault();
		}
		this.setState({ processDialogVisible: false });
	}

	hideNoEtxDialog(e) {
		if (e !== undefined) {
			e.preventDefault();
		}
		this.setState({ noExtDialogVisible: false });
	}

	hideNoHostDialog(e) {
		if (e !== undefined) {
			e.preventDefault();
		}
		this.setState({ noHostDialogVisible: false });
	}

	szafirErrorCallback(error) {
		const { onError } = this.props;
		console.log('szafirErrorCallback, error:', error);
		this.hideProcessDialog();
		// signButtonDisable(false);
		// alert('[***** Błąd Szafir SDK *****]\n\n', error);
		onError(`Błąd Szafir SDK: ${error}`);
	}

	initSzafirApplet() {
		const { devMode, devSzafirUri } = this.props;
		// eslint-disable-next-line no-undef
		const SZAFIR = SZAFIR_SDK;
		if (SZAFIR.isModeApplet()) {
			const archive = devMode
				? `${devSzafirUri}/o/rest-api/szafir/cryptoapplet.jar`
				: 'o/rest-api/szafir/cryptoapplet.jar';
			const libDir = devMode ? `${devSzafirUri}/o/rest-api/szafir/` : '/o/rest-api/szafir';

			document.getElementById(
				'szafirApplet'
			).innerHTML = `${`<applet name="CA" code="pl.com.kir.crypto.applet.CryptoApplet" archive="${archive}" WIDTH="0" HEIGHT="0">\n` +
				'<param name="Server-Library" value="'}${libDir}"/>Your browser does not understand Java.</applet>`;
		}
	}

	signEPUAP() {
		const { asyncMode, asyncPrepareDataToSign, dataToSignB64, onError } = this.props;
		console.log('signEPUAP...');
		let dataToSignBase64 = undefined;
		if (asyncMode) {
			this.setState({ processDialogVisible: true });
			const promise = asyncPrepareDataToSign();
			if (promise === undefined) {
				this.hideProcessDialog();
				return;
			}
			promise
				.then(data => {
					console.log('signEPUAP, dataToSign:', data.dataToSign);
					dataToSignBase64 = data.dataToSign;
					this.setState({ dataToSignB64: dataToSignBase64 }, () => this.epuapAddDocumentToSign());
				})
				.catch(() => {
					this.hideProcessDialog();
					onError('Błąd generowania danych do podpisu');
				});
		} else {
			dataToSignBase64 = dataToSignB64;
			this.setState(
				{
					processDialogVisible: true,
					dataToSignB64: dataToSignBase64,
				},
				() => this.epuapAddDocumentToSign()
			);
		}
	}

	epuapRedirect(epuapResult) {
		console.log('epuapRedirect, epuapResult:', epuapResult);
		window.location.href = epuapResult.epuapSignUrl;
	}

	epuapAddDocumentToSign() {
		const { element, onError } = this.props;
		const { dataToSignB64 } = this.state;
		console.log('epuapAddDocumentToSign...');
		const browserBackUrl = window.location.href;
		const elementB64 = btoa(unescape(encodeURIComponent(JSON.stringify(element))));
		console.log('epuapAddDocumentToSign, elementB64:', elementB64);
		const data = {
			documentB64: dataToSignB64,
			browserBackUrl,
			cachedData: elementB64,
		};
		this.service
			.addDocument(data)
			.then(result => this.epuapRedirect(result))
			.catch(err => {
				console.log('error:', err);
				onError('Wystąpił nieoczekiwany błąd podpisu EPUAP');
				this.hideProcessDialog();
			});
	}

	handleSign() {
		console.log('handleSign...');
		this.signQualifyCerificate();
	}

	renderProgressDialog() {
		const { id } = this.props;
		const { processDialogVisible } = this.state;
		return (
			<Dialog
				ariaCloseIconLabel="Zamknij okno dialogowe"
				key={`${id}-dialog`}
				visible={processDialogVisible}
				style={{ width: '100vw', height: '10vh', top: '0vh' }}
				modal
				closable={false}
				header={null}
				className="file-upload-dialog"
				onHide={() => this.hideProcessDialog()}
			>
				<ProgressSpinner className="file-upload-spinner" />
			</Dialog>
		);
	}

	renderNoSzafirExtDialog() {
		const { id } = this.props;
		const { noExtDialogVisible } = this.state;
		const noExtDialogFooter = (
			<div>
				<Button label="Zamknij" icon="pi pi-check" onClick={e => this.hideNoEtxDialog(e)} />
			</div>
		);
		return (
			<Dialog
				ariaCloseIconLabel="Zamknij okno dialogowe"
				key={`${id}-noExtDialog`}
				visible={noExtDialogVisible}
				style={{ width: '40vw' }}
				modal
				closable
				header="Ostrzeżenie - Brak rozszerzenia Szafir SDK"
				onHide={() => this.hideNoEtxDialog()}
				footer={noExtDialogFooter}
			>
				Niniejsza strona wykorzystuje komponenty podpisu elektronicznego&nbsp;
				<a tabIndex="0" href="http://elektronicznypodpis.pl/" target="_blank">
					Szafir SDK
				</a>
				&nbsp;
				<a tabIndex="0" href="http://www.kir.pl/" target="_blank">
					Krajowej Izby Rozliczeniowej S.A.
				</a>
				<br />W wersji dla przeglądarek <b>Google Chrome</b>, <b>Opera</b> i&nbsp;
				<b>Mozilla Firefox od wersji 52</b> wymagana jest instalacja dedykowanego rozszerzenia&nbsp;
				<b>Podpis elektroniczny Szafir SDK</b> oraz aplikacji udostępniającej funkcje podpisu
				elektronicznego.
				<p id="szafirsdk-link-buttons">
					<a
						tabIndex="0"
						id="szafirsdk-link-extension"
						// eslint-disable-next-line no-undef
						href={SZAFIR_SDK.getExtLink()}
						target="_blank"
					>
						Instalacja rozszerzenia z Web Store
					</a>
				</p>
				<div>
					<b>Uwaga:</b> Po zainstalowaniu rozszerzenia należy przeładować bieżącą stronę!
				</div>
			</Dialog>
		);
	}

	renderNoSzafirHostDialog() {
		const { id } = this.props;
		const { noHostDialogVisible } = this.state;
		const noHostDialogFooter = (
			<div>
				<Button label="Zamknij" icon="pi pi-check" onClick={e => this.hideNoHostDialog(e)} />
			</div>
		);
		let appLinkWin = undefined;
		let appLinkLinux = undefined;
		// eslint-disable-next-line no-undef
		const SZAFIR = SZAFIR_SDK;
		if (SZAFIR.getAppLinks() !== undefined) {
			appLinkWin = SZAFIR.getAppLinks().windows;
			appLinkLinux = SZAFIR.getAppLinks().linux;
		}
		return (
			<Dialog
				ariaCloseIconLabel="Zamknij okno dialogowe"
				key={`${id}-noHostDialog`}
				visible={noHostDialogVisible}
				style={{ width: '40vw' }}
				modal
				closable
				header="Ostrzeżenie - Brak aplikacji Szafir Host"
				onHide={() => this.hideNoHostDialog()}
				footer={noHostDialogFooter}
			>
				{!SZAFIR.isHostUpdateRequired() ? (
					<div id="szafirsdk-hostapp-install">
						Przeglądarka wymaga zainstalowania dedykowanej aplikacji&nbsp;
						<b>Szafir Host</b> umożliwiającej składanie podpisu elektronicznego.
					</div>
				) : null}

				{SZAFIR.isHostUpdateRequired() ? (
					<div id="szafirsdk-hostapp-update" style={{ color: 'Red' }}>
						Posiadasz nieaktualną wersję aplikacji <b>Szafir Host</b>&nbsp; używaną przez
						przeglądarkę do składania podpisu elektronicznego!!!
					</div>
				) : null}
				<p id="szafirsdk-nohost-link-buttons">
					<a tabIndex="0" style={{ maringBottom: '2px' }} href={appLinkWin} target="_blank">
						Pobierz Szafir Host dla Windows
					</a>
					<br />
					<a
						tabIndex="0"
						id="szafirsdk-link-installer-ux"
						href={appLinkLinux}
						style={{ maringBottom: '2px' }}
						target="_blank"
					>
						Pobierz Szafir Host dla Linux/Mac OS
					</a>
				</p>
				<div>
					<b>Uwaga:</b> Po zainstalowaniu aplikacji Szafir Host należy przeładować bieżącą stronę!
				</div>
				<div
					style={{ backgroundColor: '#78BED7', color: 'White', padding: '4px', marginTop: '4px' }}
				>
					Jeżeli aplikacja została prawidłowo zainstalowana i mimo tego nadal&nbsp; pojawia się
					niniejszy komunikat, należy sprawdzić czy na komputerze&nbsp; zainstalowane jest
					środowisko uruchomieniowe Java JRE.
					<br />
					Środowisko Java JRE można pobrać i zainstalować z lokalizacji:&nbsp;
					<a tabIndex="0" href="https://www.java.com/download/" target="_blank">
						Java JRE
					</a>
				</div>
			</Dialog>
		);
	}

	onButtonClick(event) {
		if (event) {
			event.preventDefault();
		}
		this.menuRef.current.toggle(event);
	}

	render() {
		const { className, label, menuButtonSize, menuButtonVariant, menuMode, rendered } = this.props;
		const { items } = this.state;
		// console.log('+++options:', options, ', epuapOptionEnabled:', epuapOptionEnabled, ', certOptionEnabled:', certOptionEnabled);
		return rendered ? (
			<span>
				{this.renderProgressDialog()}
				{this.renderNoSzafirExtDialog()}
				{this.renderNoSzafirHostDialog()}
				<Menu model={items} popup ref={this.menuRef} appendTo={document.body} />
				<ActionButton
					label={label}
					className={className}
					handleClick={this.onButtonClick.bind(this)}
					rendered={menuMode}
					size={menuButtonSize}
					variant={menuButtonVariant}
				/>
				{!menuMode && items !== undefined && items.length > 0
					? items.map(item => {
							return (
								<ActionButton label={item.label} className={className} handleClick={item.command} />
							);
					  })
					: null}
			</span>
		) : null;
	}
}

SignButtonComponent.defaultProps = {
	debug: false,
	devMode: false,
	devSzafirUri: 'http://localhost:8088',
	label: 'Podpisz',
	menuButtonSize: 'normal',
	menuButtonVariant: 'dark',
	menuMode: false,
	asyncMode: false,
	rendered: true,
	options: 'CERT|EPUAP',
};

SignButtonComponent.propTypes = {
	asyncMode: PropTypes.bool,
	asyncPrepareDataToSign: PropTypes.func,
	baseRestApiUrl: PropTypes.string.isRequired,
	className: PropTypes.string,
	dataToSignB64: PropTypes.string,
	debug: PropTypes.bool,
	devMode: PropTypes.bool,
	devSzafirUri: PropTypes.string,
	element: PropTypes.object,
	epuapResponseId: PropTypes.string,
	epuapResult: PropTypes.string,
	handleEpuapResult: PropTypes.func,
	id: PropTypes.string.isRequired,
	label: PropTypes.string,
	menuButtonSize: PropTypes.string,
	menuButtonVariant: PropTypes.string,
	menuMode: PropTypes.bool,
	onDataSigned: PropTypes.func.isRequired,
	onError: PropTypes.func.isRequired,
	options: PropTypes.string, // CERT, EPUAP, DOWNLOADXML, UPLOADSIGNATURE
	rendered: PropTypes.bool,
};

export default SignButtonComponent;
